<?php
namespace Webshop\Options;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\DelegatorFactoryInterface;

/**
 * Class OptionsAwareDelegatorFactory
 *
 * @package Webshop\Options
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/webshop-options
 */
final class OptionsAwareDelegatorFactory implements DelegatorFactoryInterface
{
    /**
     * A factory that creates delegates of a given service
     *
     * @param ContainerInterface $container
     * @param string $name
     * @param callable $callback
     * @param null|array $options
     * @return OptionsAwareInterface
     */
    public function __invoke(ContainerInterface $container, $name, callable $callback, array $options = null): OptionsAwareInterface
    {
        $instance = $callback();

        if (!$instance instanceof OptionsAwareInterface) {
            throw new ServiceNotCreatedException(sprintf(
                "Klasa mora implementirati '%s', a dobio sam '%s'.",
                OptionsAwareInterface::class,
                is_object($instance) ? get_class($instance) : gettype($instance)
            ));
        }

        $options = $container->get(Service\OptionManagerInterface::class);
        $instance->setOptionManager($options);

        return $instance;
    }
}
